<?php
define('INIT','');
define('VERSION', '0.9.2');
require('config.php');

if (DEBUG)
{
	function my_error_handler($errno)
	{
		if (error_reporting() & $errno)
		// Error-Reporting-Direktive beachten
		{
			ob_start();
			print_r(debug_backtrace());
			echo nl2br(str_replace(' ','&nbsp;',htmlentities(ob_get_clean())));
		}
		return FALSE;
	}
	set_error_handler('my_error_handler');
}

function error_file($info)
{
	$now = time();
	$n = 1;
	do
	{
		$filename = 'cron-error/'.$now.($n>1 ? '-'.$n : '').'.txt';
		$file = @fopen($filename,'x');
		$n++;
		// Schleife abbrechen, wenn Datei erfolgreich angelegt wurde
		// Wenn Datei nicht existiert, ist das Verzeichnis offenbar schreibgeschützt
		// Auch in diesem Fall Schleife abbrechen
	} while (!$file and file_exists($filename));
	if ($file)
	{
		fwrite($file, $info);
		fclose($file);
		return true;
	}
	return false;
}

function db_connect_error()
{
	error_file('Konnte keine Verbindung zur Datenbank aufbauen.');
	if (DEBUG)
	{
		ob_start();
		print_r($e);
		echo nl2br(str_replace(' ','&nbsp;',htmlentities(ob_get_clean())));
	}
	exit;
}

require('db.php');
require('functions.php');

if ($g_settings['cronlast']+$g_settings['cronmin'] > time())
{
	// Script beenden, wenn es noch nicht Zeit ist für den nächsten Durchlauf
	exit;
}

require('smtpLib.php');
$mailer = new mail_send_wrapper;
require('imapLib.php');
$imap = new imap_mail;
require($g_settings['weblib']);
$weblib = WEBLIB;
$weblib = new $weblib;

function error_mail($info, $send_via, $notify)
{
	global $g_db, $mailer;
	$sent = array();
	$fail = array();
	try
	{
		if (trim($notify) == '')
			// Keine Empfänger -> es muss keine Mail gesendet werden, alles ist bestens
			return true;
		$subject = 'Rosmarin: Cron-Fehler';
		
		if ($send_via == 0)
		{
			$smtp = false;
		}
		else
		{
			//SMTP-Server benutzen
			$dbs = $g_db->prepare('SELECT addr, port, encryption, authentication, certcheck, username, password, sender FROM `'.DB_PRE.'smtp` WHERE id=?');
			if (!$dbs->execute(array($send_via))) return false;
			if (!$smtp = $dbs->fetch()) return false;
		}
		
		$mailer->disconnect();
		if ($mailer->connect($smtp))
		{
			foreach (explode(',', $notify) as $to)
			{
				$to = trim($to);
				if ($mailer->send($to, $subject, $info, true))
					$sent[] = $to;
				else
					$fail[] = $to;
			}
			$mailer->disconnect();
		}
		if ($fail)
		{
			if ($sent) return $fail;
			else return false;
		}
		else return true;
	}
	catch (PDOException $e)
	{
		return false;
	}
}

function admin_error_mail($info)
{
	global $g_db, $g_default_values, $g_settings;
	try
	{
		$dbs = $g_db->query('SELECT notify_addr, mail_send_via FROM `'.DB_PRE.'groups` WHERE id=0');
		$mail_data = $dbs->fetch();
	}
	// Exception verwerfen
	catch (PDOException $e)
	{
		$mail_data = false;
	}
	
	if ($mail_data)
	{
		$mail_data['mail_send_via'] = standard($mail_data['mail_send_via'], $g_default_values['mail_send_via']);
		
		$info.= "\r\n\r\n".'Link zur Weboberfläche: '.$g_settings['siteurl'];
		
		// Info an Admin-Gruppe
		$c = error_mail('Warnung an die Admin-Gruppe:'."\r\n".$info, $mail_data['mail_send_via'], $mail_data['notify_addr']);
		
		if (is_array($c))
		{
			// Für einige Empfänger konnten keine Mails versendet werden
			// Diejenigen Empfänger, für die ein Versand möglich ist, über das Problem informieren
			$fail_list = implode(', ',$c);
			$c = error_mail('Warnung an die Admin-Gruppe:'."\r\n".'Admin-Benachrichtigung konnte nicht zugestellt werden für '.$fail_list.'.', $mail_data['mail_send_via'], $mail_data['notify_addr']);
		}
	}
	
	if (!$mail_data or $c === false)
	{
		// Mailversand für alle Empfänger fehlgeschlagen
		// Info an Admin-Gruppe
		error_file($info."\r\n".'Die Admin-Gruppe konnte nicht per Mail über das Problem informiert werden.', $mail_data['mail_send_via'], $mail_data['notify_addr']);
	}
}

try
{
	if ($g_settings['cronlast']+$g_settings['cronmax'] < time())
	{
		// Cronjob ist zu lange nicht gelaufen
		admin_error_mail('Cronjob wurde für länger als '.((int)($g_settings['cronmax']/60)).' Minuten nicht aufgerufen. Letzter Aufruf: '.date($g_settings['dateformat'],$g_settings['cronlast']));
	}
	$dbs = $g_db->prepare('UPDATE `'.DB_PRE.'settings` SET value=? WHERE `key`=\'cronlast\'');
	$dbs->execute(array(time()));
	
	// Hartkodierte Standardwerte mit denen der Admin-Gruppe überschreiben
	$dbs = $g_db->query('SELECT web_timeout, web_retries, web_redirects, web_check_statuscode, web_check_content, web_cert_check, mail_timeout, mail_send_via FROM `'.DB_PRE.'groups` WHERE id=0');
	$row = $dbs->fetch();
	foreach ($row as $key=>$value)
	{
		if ($value != -1)
		{
			$g_default_values[$key] = $value;
		}
	}
	
	$dbs = $g_db->query('SELECT id, name, web_timeout, web_retries, web_redirects, web_check_statuscode, web_check_content, web_cert_check, mail_timeout, mail_send_via, notify_addr, notify_pause, notify_lastsent FROM `'.DB_PRE.'groups`');
	$grouplist = $dbs->fetchAll();
	foreach ($grouplist as $group)
	{
		// Standardwerte verwenden, wenn gewünscht
		$may_use_defaults = array('web_timeout', 'web_retries', 'web_redirects', 'web_check_statuscode', 'web_check_content', 'web_cert_check', 'mail_timeout', 'mail_send_via');
		foreach ($group as $key=>$value)
		{
			if (in_array($key, $may_use_defaults) and $value == -1)
			{
				$group[$key]=$g_default_values[$key];
			}
		}
		
		//***************
		//*     Web     *
		//***************
		$report_web = array();
		
		$dbs = $g_db->prepare('SELECT id, url, content, `group`, timeout, redirects, retries, check_statuscode, check_content, cert_check, comment FROM `'.DB_PRE.'webtest` WHERE `group`=? AND active=1');
		$dbs->execute(array($group['id']));
		while ($row = $dbs->fetch())
		{
			$test_info[$row['id']] = array(
				'url' => $row['url'],
				'comment' => $row['comment'],
				'group' => $row['group'],
				'content' => $row['content'],
				);
			$weblib->add(
				$row['id'],
				$row['url'],
				standard($row['timeout'], $group['web_timeout']),
				standard($row['redirects'], $group['web_redirects']),
				standard($row['retries'], $group['web_retries']),
				$row['content'],
				standard($row['check_statuscode'], $group['web_check_statuscode']),
				standard($row['check_content'], $group['web_check_content']),
				standard($row['cert_check'], $group['web_cert_check'])
				);
		}
		
		$dbs = $g_db->prepare('UPDATE `'.DB_PRE.'webtest` SET cache_content=?, cache_time=? WHERE id=?');
		foreach($weblib->exec() as $tested)
		{
			$dbs->execute(array(end($tested['result']),time(),$tested['id']));
			$row = $test_info[$tested['id']];
			
			$exp_found = 'Erwartet: '.$row['content']."\n";
			foreach ($tested['result'] as $n=>$result)
			{
				if ($n==0)
					$exp_found .= 'Gefunden: '.$result."\n";
				else
					$exp_found .= '          '.$result."\n";
			}

			logfile($tested['id'], 'web', 0, ($row['content']!=end($tested['result']) and $row['content']!=''), $row['group'], $row['url'].($row['comment'] ? ' ('.$row['comment'].')' : ''), $exp_found."\n".$tested['log']);
			if ($row['content']!=end($tested['result']) and $row['content']!='')
			{
				// Nicht das erwartete Ergebnis
				$report_web[$tested['id']] = 'Webtest '.$row['url'].($row['comment'] ? ' ('.$row['comment'].')' : '')."\r\n   ".substr(str_replace("\n", "\r\n   ", $exp_found),0,-3);
			}
		}
		// Curl-Multi liefert die Ergebnisse nicht geordnet.
		// Dies ist aber notwendig, um mit den Ergebnissen vom letzten mal
		// vergleichen zu können.
		ksort($report_web);
		
		//****************
		//*     Mail     *
		//****************
		$report_mail = array();
		$dbs = $g_db->prepare('SELECT id, `group`, path, username, password, timeout, send_via, comment FROM `'.DB_PRE.'mailtest` WHERE `group`=? AND active=1 ORDER BY id');
		$dbs->execute(array($group['id']));
		while ($mailtest_list = $dbs->fetchAll())
		foreach ($mailtest_list as $mailtest)
		{
			$mailerror = array();
			
			//Mails senden
			$send_via = standard($mailtest['send_via'], $group['mail_send_via']);
			$dbs = $g_db->prepare('SELECT id, address FROM `'.DB_PRE.'mail_sendto` WHERE tid=?');
			$dbs->execute(array($mailtest['id']));
			$sendto_list = array();
			while ($row = $dbs->fetch())
			{
				$sendto_list[$row['id']] = $row['address'];
			}
			if ($send_via == 0)
			{
				$smtp=false;
			}
			else
			{
				//SMTP-Server benutzen
				$dbs = $g_db->prepare('SELECT addr, port, encryption, authentication, username, password, certcheck, sender FROM `'.DB_PRE.'smtp` WHERE id=?');
				$dbs->execute(array($send_via));
				$smtp = $dbs->fetch();
			}
			
			$send_error = false;
			$mailer->disconnect();
			if (!$mailer->connect($smtp))
			{
				$mailerror[] = 'Verbindung zum SMTP-Server fehlgeschlagen: '.$mailer->geterror();
				$send_error = true;
			}
			// Mails auch versuchen zu versenden, wenn connect()
			// fehlschlägt, damit eine Warnung verschickt wird
			
			$dbs = $g_db->prepare('INSERT INTO `'.DB_PRE.'mail_sent` (aid, tid, time, hash) VALUES (?,?,?,?)');
			foreach($sendto_list as $sendto_id=>$sendto_addr)
			{
				while (true)
				{
					try
					{
						$mail_hash=rand_str(15);
						$dbs->execute(array($sendto_id, $mailtest['id'], time(),$mail_hash));
						// Wenn hash schon vergeben, dann wird Exception geworfen
						// break wird nicht ausgeführt -> weiterer Schleifendurchlauf
						break;
					}
					catch (PDOException $e)
					{
						// Wenn Code == 1062 (Duplicate Entry), dann wird 
						// die normale Fehlerbehandlung nicht durchgeführt
						if ($e->errorInfo[1]!=1062) throw $e;
					}
				}
				if (!$mailer->send($sendto_addr, 'Mailtest', $mail_hash))
				{
					$mailerror[] = $sendto_addr.': Mail konnte nicht gesendet werden.';
					$send_error = true;
				}
				$mailer->disconnect();
			}
			if ($log = $mailer->getlog())
			{
				logfile($mailtest['id'], 'mail', 0, $send_error, $mailtest['group'], 'S: '.implode(', ',$sendto_list).($mailtest['comment'] ? ' ('.$mailtest['comment'].')' : ''), $log);
			}
			
			
			// Mails abrufen
			$dbs = $g_db->prepare('SELECT hash FROM `'.DB_PRE.'mail_sent` WHERE tid=?');
			$dbs->execute(array($mailtest['id']));
			$hashes = $dbs->fetchAll(PDO::FETCH_COLUMN);
			
			$received = $imap->check($mailtest['path'], $mailtest['username'], $mailtest['password'], $hashes);
			if ($received === FALSE)
			{
				$mailerror[] = 'Mails konnten nicht abgerufen werden'.($imap->geterror() ? ': '.$imap->geterror() : '');
			}
			else
			{
				$dbs = $g_db->prepare('UPDATE `'.DB_PRE.'mailtest` SET lastcheck=? WHERE id=?');
				$dbs->execute(array(time(),$mailtest['id']));
				if (!empty($received))
				{
					$dbs = $g_db->query('DELETE FROM `'.DB_PRE.'mail_sent` WHERE hash IN '.in_statement($received, $g_db));
				}
			}
			logfile($mailtest['id'], 'mail', 1, ($received === FALSE), $mailtest['group'], 'R: '.implode(', ',$sendto_list).($mailtest['comment'] ? ' ('.$mailtest['comment'].')' : ''), $imap->getlog());
			
			$dbs = $g_db->prepare('SELECT COUNT(*) FROM `'.DB_PRE.'mail_sent` WHERE tid=? AND time<?');
			$timeout = standard($mailtest['timeout'], $group['mail_timeout']);
			$dbs->execute(array($mailtest['id'], time()-$timeout));
			if ($dbs->fetchColumn())
			{
				$mailerror[] = 'Einige Emails sind nach über '.((int)($timeout/60)).' Minuten immer noch nicht angekommen';
			}
			
			if ($mailerror)
			{
				$report_mail[$mailtest['id']] = 'Mailtest '.$mailtest['path'].' / '.$mailtest['username']."\r\n   ".implode("\r\n   ",$mailerror)."\r\n";
			}
		}
		$report = '';
		if ($report_web) $report.= implode("\r\n",$report_web);
		if ($report_web and $report_mail) $report.= "\r\n";
		if ($report_mail) $report.= implode("\r\n",$report_mail);
		
		$last_report = @file_get_contents('cron-data/'.$group['id'].'.txt');
		file_put_contents('cron-data/'.$group['id'].'.txt',$report);
		if ($report != $last_report or ($report and $group['notify_lastsent']+$group['notify_pause'] < time()))
		{
			//Es muss eine Email gesendet werden
			if (!$report)
			{
				// Letzter Report enthielt Fehlermeldungen, dieser aber nicht
				$report = 'Alle Fehler behoben';
			}
			else
			{
				$report = 'Serverüberwachung: Gruppe '.$group['name']."\r\n\r\n".$report;
			}
			$report.= "\r\n\r\n".'Link zur Weboberfläche: '.$g_settings['siteurl'];
			$c = error_mail($report, $group['mail_send_via'], $group['notify_addr']);
			if ($c !== false)
			{
				// Mindestens eine Mail konnte gesendet werden
				// Eintrag in Datenbank
				$dbs = $g_db->prepare('UPDATE `'.DB_PRE.'groups` SET notify_lastsent=? WHERE id=?');
				$dbs->execute(array(time(), $group['id']));
			}
			if (is_array($c))
			{
				// Für einige Empfänger konnten keine Mails versendet werden
				// Diejenigen Empfänger, für die ein Versand möglich ist, über das Problem informieren
				$fail_list = implode(', ',$c);
				$c = error_mail('Gruppe '.$group['name'].': Gruppenbenachrichtigung konnte nicht zugestellt werden für '.$fail_list.'.', $group['mail_send_via'], $group['notify_addr']);
			}
			if ($c === false)
			{
				// Mailversand für alle Empfänger fehlgeschlagen
				// Info an Admin-Gruppe
				admin_error_mail('Mailversand für Gruppe '.$group['name'].' nicht möglich');
			}
		}
	}
}
catch (PDOException $e)
{
	ob_start();
	echo "Es ist ein Datenbank-Problem aufgetreten:\n\n";
	print_r($e);
	$ob = ob_get_clean();
	if (DEBUG)
	{
		echo nl2br(str_replace(' ','&nbsp;',htmlentities($ob)));
	}
	admin_error_mail($ob);
	exit;
}
