<!DOCTYPE html>
<html>
<head>
	<meta charset="utf-8"> 
	<link rel="stylesheet" type="text/css" href="style.css">
	<title>Rosmarin: Installation</title>
</head>
<body class="install">
	<div class="parbox">
		<span class="parhead">Installation</span>
		<div class="parbody">

<?php
define('INIT','');
require_once('config.php');
require_once('functions.php');

if ( (!empty($_POST['ready'])) or (isset($_POST['username']) and isset($_POST['password']) and isset($_POST['password2'])) )
{
	// Aus Sicherheitsgründen keine Installation durchführen, wenn schon Tabellen vorhanden sind
	try {
		$db = new PDO('mysql:host='.DB_HOST.';dbname='.DB_NAME, DB_USER, DB_PASS);
		$db->setAttribute(PDO::ATTR_ERRMODE, PDO::ERRMODE_EXCEPTION);
		$db->setAttribute(PDO::ATTR_DEFAULT_FETCH_MODE, PDO::FETCH_ASSOC);
		$db->exec('SET NAMES UTF8');
		$db->exec("SET sql_mode=''");
		$dbs = $db->prepare("SELECT COUNT(*) FROM information_schema.`TABLES` WHERE `TABLE_SCHEMA` LIKE BINARY ? AND `TABLE_NAME` LIKE BINARY ?");
		$dbs->execute(array(DB_NAME, DB_PRE.'%'));
		$table_count = $dbs->fetchColumn();
	} catch(PDOException $e) {
		echo 'Datenbankfehler';
		exit;
	}
	
	if ($table_count != 0)
	{
		$step = 'dbcommand';
	}
	else
	{
		if (!empty($_POST['ready']))
		{
			$step = 'logindata';
		}
		elseif (!$_POST['username'] or !$_POST['password'])
		{
			$errormsg = 'Benutzername und Passwort dürfen nicht leer sein';
			$step = 'logindata';
		}
		elseif ($_POST['password'] != $_POST['password2'])
		{
			$errormsg= 'Die Passwörter stimmen nicht überein';
			$step = 'logindata';
		}
		else
		{
			$step = 'install';
		}
	}
}
else
{
	$step = 'test';
}


if ($step=='test')
{
	# Schritt 1: Tests
	
	$tests = array();
	
	$tests[]= array('PHP-Version 5.1 oder höher', 'Rosmarin wird mit PHP-Versionen kleiner als 5.1 nicht funktionieren', (strnatcmp(PHP_VERSION, '5.1') >= 0));
	$tests[]= array('Schreibzugriff auf die Verzeichnisse cron-data, cron-error und logs', 'Rosmarin wird ohne Schreibzugriff nicht funktionieren', (is_writeable('cron-data') and is_writeable('cron-error') and is_writeable('logs')));
	
	try  {
		$db = new PDO('mysql:host='.DB_HOST.';dbname='.DB_NAME, DB_USER, DB_PASS);
		$db->setAttribute(PDO::ATTR_ERRMODE, PDO::ERRMODE_EXCEPTION);
		$db->setAttribute(PDO::ATTR_DEFAULT_FETCH_MODE, PDO::FETCH_ASSOC);
		$db->exec('SET NAMES UTF8');
		$db->exec("SET sql_mode=''");
	} catch(PDOException $e) {
		$db = false;
	}
	$tests[]= array('Verbindung zur Datenbank möglich', 'Ohne Datenbankverbindung wird Rosmarin nicht funktionieren', (bool)$db);
	
	if ($db) {
		try {
			$dbs = $db->query("SHOW VARIABLES LIKE 'version'");
			$mysql_version = $dbs->fetchColumn(1);
		} catch(PDOException $e) {
			$mysql_version = 0;
		}
		$tests[]= array('MySQL-Server-Version 5.0 oder höher', 'Rosmarin wird mit MySQL-Server-Version kleiner als 5.0 nicht funktionieren', (strnatcmp($mysql_version, '5.0') >= 0));
		
		try {
			$dbs = $db->prepare("SELECT COUNT(*) FROM information_schema.`TABLES` WHERE `TABLE_SCHEMA` LIKE BINARY ? AND `TABLE_NAME` LIKE BINARY ?");
			$dbs->execute(array(DB_NAME, DB_PRE.'%'));
			$table_count = $dbs->fetchColumn();
		} catch(PDOException $e) {
			$table_count = -1;
		}
		$tests[]= array('Das gewählten Datenbank-Präfix wird bisher nicht verwendet', 'Falls bereits Tabellen mit diesem Präfix vorhanden sind, müssen diese Tabellen zunächst gelöscht werden', ($table_count == 0));
	}
	
	$tests[]= array('CURL-Erweiterung verfügbar', 'Ohne CURL sind die Webtests nicht voll funktionsfähig', extension_loaded('curl'));
	$tests[]= array('allow_url_fopen aktiv', 'Dient als Ersatz für CURL. Wenn weder Curl noch allow_url_fopen verfügbar sind, werden die Webtests nicht funktionieren', ini_get('allow_url_fopen'));
	$tests[]= array('IMAP-Erweiterung verfügbar', 'Ohne IMAP werden die Mailtests nicht funktionieren', extension_loaded('imap'));
	$tests[]= array('Zertifizierungsstellen-Datei vorhanden', 'Ohne eine Zertifizierungsstellen-Datei können keine SSL-Zertifikate überprüft werden', is_readable(CERTFILE));
	
	?>
	<p>Es wurden einige Tests durchgeführt. Im Idealfall sollte das Ergebnis bei allen Tests ‚ja‘ lauten. Wenn dies nicht der Fall ist, sollten Sie die gefundenen Probleme beheben, bevor sie die Installation fortsetzen.</p>
	<p>Sie können die Installation auch fortsetzen, wenn die Testergebnisse nicht zufriedenstellend sind. Rosmarin wird dann aber nur eingeschränkt oder gar nicht funktionsfähig sein.</p>
	<table>
		<tr><th>Test</th><th>Ergebnis</th></tr>
		<?php foreach ($tests as $t) { ?>
			<tr class="<?php echo $t[2] ? 'ok': 'error'; ?>">
				<td>
					<div><?php echo htmlspecialchars($t[0]); ?></div>
					<div class="explain"><?php echo htmlspecialchars($t[1]); ?></div>
				</td>
				<td>
					<?php echo $t[2] ? 'ja': 'nein'; ?>
				</td>
			</tr>
		<?php } ?>
	</table>
	
	<form method="get">
		<input type="submit" value="erneut prüfen">
		<input type="submit" name="submit" value="weiter &gt;" form="form2">
	</form>
	<form method="post" id="form2">
		<input type="hidden" name="ready" value="y">
	</form>
	<?php
}
elseif ($step == 'dbcommand')
{
	# Schritt 2: Präfix-Problem
	?>
	<p><strong>Achtung:</strong> In der Datenbank existieren bereits Tabellen mit dem Präfix &quot;<?php echo htmlspecialchars(DB_PRE); ?>&quot;. Es wird empfohlen, ein anderes Präfix zu wählen und die Installation neu zu starten. Wenn Sie die Installation mit diesem Datenbank-Präfix fortsetzen wollen, müssen Sie zunächst die vorhandenen Tabellen löschen. Führen Sie dazu folgenden SQL-Befehl aus:</p>
	<p><code><?php
		try  {
			$dbs = $db->prepare("SELECT `TABLE_NAME` FROM `information_schema`.`TABLES` WHERE `TABLE_SCHEMA` LIKE BINARY ? AND `TABLE_NAME` LIKE BINARY ?");
			$dbs->execute(array(DB_NAME, DB_PRE.'%'));
			$table_list = $dbs->fetchAll(PDO::FETCH_COLUMN);
		} catch(PDOException $e) {
			echo 'Datenbankfehler';
			exit;
		}
		echo htmlspecialchars('USE `'.DB_NAME.'`; DROP TABLE IF EXISTS `'.implode('`, `', $table_list).'`;');
	?></code></p>
	<p>Aus Sicherheitsgründen müssen Sie diese SQL-Befehl selbst ausführen (z.B. mit phpMyAdmin).</p>
	<form method="get">
		<input type="submit" value="Installation neu starten">
	</form>
	<?php
}
elseif ($step == 'logindata')
{
	# Schritt 3: Nutezrname/Passwort
	?>
	Geben Sie bitte Nutzernamen und Passwort für das Administratorkonto an. Sie können später noch weitere Administratorkonten anlegen.
	<?php if (isset($errormsg)) echo '<p>Fehler: '.$errormsg.'</p>'; ?>
	<form method="post">
		<table>
			<tr><td><label for="username">Benuterzname:</label></td><td><input type="text" name="username" size="20"<?php if (isset($_POST['username'])) echo ' value="'.htmlspecialchars($_POST['username']).'"'; ?>></td></tr>
			<tr><td><label for="password">Passwort:</label></td><td><input type="password" name="password" size="20"></td></tr>
			<tr><td><label for="password2">Passwort wiederholen:</label></td><td><input type="password" name="password2" size="20"></td></tr>
		</table>
		<input type="submit" value="Installieren" name="submit">
	</form>
	<?php
}
elseif ($step == 'install')
{
	# Schritt 4: Datenbankbefehle
	try  {
$db->query('CREATE TABLE `'.DB_PRE.'groups` (
`id` int(11) NOT NULL AUTO_INCREMENT,
`name` varchar(255) NOT NULL,
`web_timeout` int(11) NOT NULL,
`web_retries` int(11) NOT NULL,
`web_redirects` int(11) NOT NULL,
`web_check_statuscode` tinyint(1) NOT NULL,
`web_check_content` tinyint(1) NOT NULL,
`web_cert_check` int(11) NOT NULL,
`mail_timeout` int(11) NOT NULL,
`mail_send_via` int(11) NOT NULL,
`notify_addr` text NOT NULL,
`notify_pause` int(11) NOT NULL,
`notify_lastsent` int(11) NOT NULL,
PRIMARY KEY (`id`),
UNIQUE KEY `name` (`name`)
) ENGINE=MyISAM  DEFAULT CHARSET=utf8');

$db->query('CREATE TABLE `'.DB_PRE.'login_fail` (
`username` varchar(255) NOT NULL,
`count` int(11) NOT NULL,
`count_total` int(11) NOT NULL,
`time` int(11) NOT NULL,
`expire` int(11) NOT NULL,
PRIMARY KEY (`username`)
) ENGINE=MyISAM DEFAULT CHARSET=utf8');

$db->query('CREATE TABLE `'.DB_PRE.'logs` (
`id` int(11) NOT NULL AUTO_INCREMENT,
`type` varchar(30) NOT NULL,
`subtype` tinyint(4) NOT NULL,
`tid` int(11) NOT NULL,
`time` int(11) NOT NULL,
`error` tinyint(1) NOT NULL,
`group` int(11) NOT NULL,
`title` text NOT NULL,
PRIMARY KEY (`id`),
KEY `type_tid` (`type`,`tid`)
) ENGINE=MyISAM  DEFAULT CHARSET=utf8');

$db->query('CREATE TABLE `'.DB_PRE.'mailtest` (
`id` int(11) NOT NULL AUTO_INCREMENT,
`group` int(11) NOT NULL,
`path` text NOT NULL,
`username` text NOT NULL,
`password` text NOT NULL,
`active` tinyint(1) NOT NULL,
`lastcheck` int(11) NOT NULL,
`comment` text NOT NULL,
`timeout` int(11) NOT NULL,
`send_via` int(11) NOT NULL,
PRIMARY KEY (`id`)
) ENGINE=MyISAM  DEFAULT CHARSET=utf8');

$db->query('CREATE TABLE `'.DB_PRE.'mail_sendto` (
`id` int(11) NOT NULL AUTO_INCREMENT,
`tid` int(11) NOT NULL,
`address` text NOT NULL,
PRIMARY KEY (`id`),
KEY `tid` (`tid`)
) ENGINE=MyISAM  DEFAULT CHARSET=utf8');

$db->query('CREATE TABLE `'.DB_PRE.'mail_sent` (
`aid` int(11) NOT NULL,
`tid` int(11) NOT NULL,
`time` int(11) NOT NULL,
`hash` varchar(20) NOT NULL,
UNIQUE KEY `hash` (`hash`),
KEY `aid` (`aid`),
KEY `tid` (`tid`)
) ENGINE=MyISAM DEFAULT CHARSET=utf8');

$db->query('CREATE TABLE `'.DB_PRE.'session` (
`sessid` varchar(255) NOT NULL,
`uid` int(11) NOT NULL,
`expire` int(11) NOT NULL,
`active_group` int(11) NOT NULL,
PRIMARY KEY (`sessid`),
KEY `uid` (`uid`)
) ENGINE=MyISAM DEFAULT CHARSET=utf8');

$db->query('CREATE TABLE `'.DB_PRE.'settings` (
`key` varchar(255) NOT NULL,
`value` text NOT NULL,
PRIMARY KEY (`key`)
) ENGINE=MyISAM DEFAULT CHARSET=utf8');

$db->query('CREATE TABLE `'.DB_PRE.'smtp` (
`id` int(11) NOT NULL AUTO_INCREMENT,
`name` varchar(255) NOT NULL,
`group` int(11) NOT NULL,
`addr` text NOT NULL,
`port` int(11) NOT NULL,
`encryption` varchar(30) NOT NULL,
`authentication` varchar(30) NOT NULL,
`certcheck` tinyint(1) NOT NULL,
`username` text NOT NULL,
`password` text NOT NULL,
`sender` text NOT NULL,
`comment` text NOT NULL,
PRIMARY KEY (`id`),
UNIQUE KEY `name` (`name`,`group`)
) ENGINE=MyISAM  DEFAULT CHARSET=utf8');

$db->query('CREATE TABLE `'.DB_PRE.'user` (
`id` int(11) NOT NULL AUTO_INCREMENT,
`name` varchar(255) NOT NULL,
`password` text NOT NULL,
PRIMARY KEY (`id`),
UNIQUE KEY `name` (`name`)
) ENGINE=MyISAM  DEFAULT CHARSET=utf8');

$db->query('CREATE TABLE `'.DB_PRE.'usergroup` (
`uid` int(11) NOT NULL,
`gid` int(11) NOT NULL,
UNIQUE KEY `uid_gid` (`uid`,`gid`),
KEY `uid` (`uid`),
KEY `gid` (`gid`)
) ENGINE=MyISAM DEFAULT CHARSET=utf8');

$db->query('CREATE TABLE `'.DB_PRE.'webtest` (
`id` int(11) NOT NULL AUTO_INCREMENT,
`group` int(11) NOT NULL,
`url` text NOT NULL,
`active` tinyint(1) NOT NULL,
`content` text NOT NULL,
`cache_content` text NOT NULL,
`cache_time` int(11) NOT NULL,
`comment` text NOT NULL,
`timeout` int(11) NOT NULL,
`retries` int(11) NOT NULL,
`redirects` int(11) NOT NULL,
`check_statuscode` tinyint(1) NOT NULL,
`check_content` tinyint(1) NOT NULL,
`cert_check` int(11) NOT NULL,
PRIMARY KEY (`id`)
) ENGINE=MyISAM  DEFAULT CHARSET=utf8');

$db->exec("SET sql_mode='NO_AUTO_VALUE_ON_ZERO'");
$db->query("INSERT INTO `".DB_PRE."groups` (`id`, `name`, `web_timeout`, `web_retries`, `web_redirects`, `web_check_statuscode`, `web_check_content`, `web_cert_check`, `mail_timeout`, `mail_send_via`, `notify_addr`, `notify_pause`, `notify_lastsent`) VALUES(0, 'Admin', -1, -1, -1, -1, -1, -1, -1, -1, 'no@body', 0, 0)");
		$g_settings['pwcost'] = '10';
		$pw = hash_pw($_POST['password']);
$dbs = $db->prepare("INSERT INTO `".DB_PRE."user` (`id`, `name`, `password`) VALUES(1, ?, ?)");
$dbs->execute(array($_POST['username'],$pw));
$db->query("INSERT INTO `".DB_PRE."usergroup` (`uid`, `gid`) VALUES(1, 0)");

		if (extension_loaded('curl'))
		{
			if (strnatcmp(PHP_VERSION, '5.2') >= 0)
				$method = 'webLibCurlMulti.php';
			else
				$method = 'webLibCurlEasy.php';
		}
		elseif (ini_get('allow_url_fopen'))
			$method = 'webLibSimple.php';
		else
			$method = 'webLibNone.php';
			
		$url = 'http://'.$_SERVER["HTTP_HOST"].$_SERVER["SCRIPT_NAME"];
		if (substr($url, -11) == 'install.php')
			$url = substr($url, 0, -11);

$dbs = $db->prepare("INSERT INTO `".DB_PRE."settings` (`key`, `value`) VALUES
('logintime', '7200'),
('dateformat', 'd.m.Y H:i:s (T)'),
('cronmin', '300'),
('cronmax', '10800'),
('pwcost', '10'),
('weblib', ?),
('phpmail_from', 'rosmarin@localhost'),
('cronlast', '0'),
('siteurl', ?),
('login_lock_num', '5'),
('login_lock_time', '600')");
$dbs->execute(array($method, $url));

	} catch(PDOException $e) {
		echo 'Datenbankfehler ('.htmlentities($e->getFile()).' on line '.htmlentities($e->getLine()).'):<br>';
		echo htmlentities($e->getMessage());
		exit;
	}
	?>
	Installation abgeschlossen. Sie können die Datei install.php jetzt löschen.
	<?php
}
?>
		
		</div>
	</div>
</body>
</html>

