<?php
if (!defined('INIT')) exit;

// Alte Sessions löschen
$dbs = $g_db->prepare('DELETE FROM `'.DB_PRE.'session` WHERE expire<?');
$dbs->execute(array(time()));

if (isset($_POST['user']) and isset($_POST['password']))
{
	// Nutzer will sich einloggen
	// Login-Sperre komplett löschen, wenn sie abgelaufen ist und der Nutzer nicht existiert
	// Wenn der Nutzer existiert, müssen die Informationen erhalten bleiben, um beim
	// nächsten Login eine Warnung anzuzeigen.
	$dbs = $g_db->prepare('DELETE FROM `'.DB_PRE.'login_fail` WHERE expire<? AND NOT EXISTS (SELECT * FROM `'.DB_PRE.'user` AS user WHERE user.name=`'.DB_PRE.'login_fail`.username)');
	$dbs->execute(array(time()));
	
	//Prüfen, ob Loginsperre für diesen Nutzernamen vorliegt
	$dbs = $g_db->prepare('SELECT count, count_total, time FROM `'.DB_PRE.'login_fail` WHERE username=?');
	$dbs->execute(array($_POST['user']));
	$login_fail = $dbs->fetch();
	if ($login_fail !== false and $login_fail['count']>=$g_settings['login_lock_num'] and $login_fail['time']+$g_settings['login_lock_time']>time())
	{
		//Nutzer ist gesperrt, weil er zu oft ein falsches Passwort eingegeben hat
		$t_error = 'Dieser Benutzer ist für '.((int)($g_settings['login_lock_time']/60)).' Minuten gesperrt';
		require('login.template.php');
		exit;
	}
	
	$dbs = $g_db->prepare('SELECT id, name, password FROM `'.DB_PRE.'user` WHERE name=?');
	$dbs->execute(array($_POST['user']));
	$user = $dbs->fetch();
	if (($user !== FALSE) and (crypt($_POST['password'],$user['password']) === $user['password']))
	{
		// Benutzername und Passwort korrekt
		$g_userid = $user['id'];
		$g_username = $user['name'];
		list ($g_active_group, $g_active_group_name) = findgroup($g_userid,$g_db);
		
		// Zähler für fehlgeschlagene Logins anzeigen und zurücksetzen
		if ($login_fail > 0)
		{
			$t_infos[] = 'Für diesen Benutzer gab es '.$login_fail['count_total'].' fehlgeschlagene(n) Loginversuch(e).';
			$dbs = $g_db->prepare('DELETE FROM `'.DB_PRE.'login_fail` WHERE username=?');
			$dbs->execute(array($g_username));
		}
		
		if (substr($user['password'], 4, 2) != $g_settings['pwcost'])
		{
			// Passwort-Kostenfaktor wurde geändert.
			// Passwort muss neu verschlüsselt werden.
			$dbs = $g_db->prepare('UPDATE `'.DB_PRE.'user` SET password=? WHERE id=?');
			$dbs->execute(array(hash_pw($_POST['password']), $g_userid));
		}
		
		$dbs = $g_db->prepare('INSERT INTO `'.DB_PRE.'session` (sessid, uid, expire, active_group) VALUES (?,?,?,?)');
		while (true)
		{
			try
			{
				$g_sessionid = rand_str(15);
				$dbs->execute(array($g_sessionid,$g_userid,time()+$g_settings['logintime'],$g_active_group));
				// Wenn sid schon vergeben, dann wird Exception geworfen
				// break wird nicht ausgeführt -> weiterer Schleifendurchlauf
				break;
			}
			catch (PDOException $e)
			{
				// Wenn Code == 1062 (Duplicate Entry), dann wird 
				// die normale Fehlerbehandlung nicht durchgeführt
				if ($e->errorInfo[1]!=1062) throw $e;
			}
		}
		setcookie('sid',$g_sessionid);
	}
	else
	{
		// Benutzername existiert nicht oder Passwort passt nicht zum Benutzernamen
		
		// Fehlgeschlagene Logins zählen
		if (!$login_fail)
		{
			// Für diesen Benutzernamen gibt es noch keinen Datenbankeintrag
			$dbs = $g_db->prepare('INSERT INTO `'.DB_PRE.'login_fail` (username, count, count_total, time, expire) VALUES (?,1,1,?,?)');
			$dbs->execute(array($_POST['user'], time(), time()+$g_settings['login_lock_time']));
		}
		else
		{
			// Es ist ein Datenbankeintrag vorhanden
			$dbs = $g_db->prepare('REPLACE INTO `'.DB_PRE.'login_fail` (username, count, count_total, time, expire) VALUES (?,?,?,?,?)');
			$login_fail['count'] = 1 + max(0, $login_fail['count'] - (int)((time()-$login_fail['time'])/$g_settings['login_lock_time']));
			$dbs->execute(array(
				$_POST['user'],
				$login_fail['count'],
				$login_fail['count_total']+1,
				time(),
				time()+$login_fail['count']*$g_settings['login_lock_time'],
				));
		}
		
		$t_error = 'Benutzername oder Passwort falsch';
		require('login.template.php');
		exit;
	}
	unset($user);

}
elseif (isset($_COOKIE['sid']))
{
	// Nutzer ist schon eingeloggt
	$dbs = $g_db->prepare('SELECT user.id AS userid, user.name AS username, groups.id AS groupid, groups.name AS groupname FROM `'.DB_PRE.'session` AS session JOIN `'.DB_PRE.'user` AS user ON user.id = session.uid LEFT JOIN `'.DB_PRE.'groups` AS groups ON groups.id = session.active_group WHERE session.sessid=?');
	$dbs->execute(array($_COOKIE['sid']));
	$result = $dbs->fetch();
	if ($result !== FALSE)
	{
		// Session ist noch gültig
		$g_userid = $result['userid'];
		$g_username = $result['username'];

		// Wenn die aktive Gruppe des Nutzers gelöscht wird, dann ist groupid und groupname NULL
		if ($result['groupid'] === NULL)
		{
			list ($g_active_group, $g_active_group_name) = findgroup($g_userid, $g_db);
			$dbs = $g_db->prepare('UPDATE `'.DB_PRE.'session` SET expire=?, active_group=? WHERE sessid=?');
			$dbs->execute(array(time()+3600, $g_active_group, $_COOKIE['sid']));
		}
		else
		{
			$g_active_group = $result['groupid'];
			$g_active_group_name = $result['groupname'];
			$dbs = $g_db->prepare('UPDATE `'.DB_PRE.'session` SET expire=? WHERE sessid=?');
			$dbs->execute(array(time()+3600, $_COOKIE['sid']));
		}
		$g_sessionid = $_COOKIE['sid'];
	}
	unset($result);
}

if (empty($g_userid))
{
	// Keine gültige Session vorhanden
	$t_error = false;
	require('login.template.php');
	exit;
}

if (isset($_GET['logout']))
{
	$dbs = $g_db->prepare('DELETE FROM `'.DB_PRE.'session` WHERE sessid=?');
	$dbs->execute(array($_COOKIE['sid']));
	setcookie('sid',FALSE);
	$t_error = false;
	require('login.template.php');
	exit;
}

function findgroup($uid, $db)
{
	$dbs = $db->prepare('SELECT groups.id, groups.name FROM `'.DB_PRE.'groups` AS groups JOIN `'.DB_PRE.'usergroup` AS usergroup ON groups.id = usergroup.gid WHERE usergroup.uid=? ORDER BY groups.id LIMIT 2');
	$dbs->execute(array($uid));
	
	// Standard-Werte, falls ein Nutzer keine Gruppen hat
	$active_group = -1;
	$active_group_name = '';

	while ($agrp = $dbs->fetch()) {
		$active_group = $agrp['id'];
		$active_group_name = $agrp['name'];
		
		// Wenn id=0 (Admin-Gruppe), dann wird Schleife erneut durchlaufen.
		// So wird nicht standardmäßig die Admingruppe ausgewählt,
		//  falls noch eine andere Gruppe existiert.
		if ($agrp['id'] != 0) break;
	}
	return array($active_group, $active_group_name);
}
