<?php
require_once('error_buffer.php');

class smtp_mail
{
	private $conn = false;
	private $log = '';
	private $lasterror = '';
	private $addr, $port, $enc, $auth, $cert, $user, $pass, $from;
	
	public function __destruct()
	{
		$this->disconnect();
	}
	
	public function connect($addr, $port, $enc, $auth, $cert, $user, $pass, $from)
	{
		if ($this->conn) return false;

		$address = (($enc == 'ssl') ? 'ssl://' : 'tcp://').$addr.':'.$port;
		$hostname = empty($_SERVER['SERVER_NAME']) ? 'Mail Test' : $_SERVER['SERVER_NAME'];
		$errcode = NULL;
		$errstring = NULL;
		$opt = array(
			'ssl' => array(
				'verify_peer' => $cert,
				'cafile' => CERTFILE,
				'CN_match' => $addr,
				),
			);
		$context = stream_context_create($opt);
		set_error_handler('error_buffer',E_WARNING);
		$this->conn = @stream_socket_client($address, $errcode, $errstring, 5, STREAM_CLIENT_CONNECT, $context);
		restore_error_handler();
		foreach (error_buffer() as $errstr)
		{
			$this->log(trim_errormsg($errstr));
		}
		if (!$this->conn)
		{
			//keine Verbindung möglich
			$this->log('Verbindungsaufbau fehlgeschlagen', true);
			return false;
		}
		stream_set_timeout($this->conn, 5);
		if ($this->read() != '220')
		{
			// keine Begrüßung
			$this->log('Server nicht bereit', true);
			$this->disconnect();
			return false;
		}
		$this->write('HELO '.$hostname);
		if ($this->read() != '250')
		{
			$this->log('Server nicht bereit', true);
			$this->disconnect();
			return false;
		}
		if ($enc == 'starttls' or $enc == 'trystarttls')
		{
			$this->write('STARTTLS');
			if ($this->read() != '220')
			{
				// STARTTLS nicht möglich
				if ($enc == 'starttls')
				{
					$this->log('STARTTLS verweigert', true);
					$this->disconnect();
					return false;
				}
				else
				{
					$this->log('STARTTLS verweigert, fahre unverschlüsselt fort');
				}
			}
			else
			{
				set_error_handler('error_buffer',E_WARNING);
				$c_en = @stream_socket_enable_crypto($this->conn,true, STREAM_CRYPTO_METHOD_TLS_CLIENT);
				restore_error_handler();
				foreach (error_buffer() as $errstr)
				{
					$this->log(trim_errormsg($errstr));
				}
				if (!$c_en)
				{
					//STARTTLS fehlgeschlagen
					$this->log('Verschlüsselung fehlgeschlagen', true);
					$this->close();
					return false;
				}
				else
				{
					$this->log('Verbindung verschlüsselt');
					$this->write('HELO '.$hostname);
					if ($this->read() != '250')
					{
						$this->log('Server nicht bereit', true);
						$this->disconnect();
						return false;
					}
				}
			}
		}
		if ($auth == 'plain')
		{
			$this->write('AUTH PLAIN '.base64_encode($user.chr(0).$user.chr(0).$pass),'AUTH PLAIN *Zensiert*');
			if ($this->read() != '235')
			{
				//Login fehlgeschlagen
				$this->log('Anmeldung fehlgeschlagen',true);
				$this->disconnect();
				return false;
			}
		}
		$this->addr = $addr;
		$this->port = $port;
		$this->enc = $enc;
		$this->auth = $auth;
		$this->cert = $cert;
		$this->user = $user;
		$this->pass = $pass;
		$this->from = $this->clean_str($from);
		return true;
	}
	
	public function send($to, $subject, $body, $headers)
	{
		$to = $this->clean_str($to);
		$subject = $this->clean_str($subject);
		$body = $this->clean_str($body, true);
		$this->write('MAIL FROM:<'.$this->from.'>');
		if (substr($this->read(),0,2) != '25')
		{
			$this->reset();
			return false;
		}
		$this->write('RCPT TO:<'.$to.'>');
		if (substr($this->read(),0,2) != '25')
		{
			$this->reset();
			return false;
		}
		$this->write('DATA');
		if ($this->read() != '354')
		{
			$this->reset();
			return false;
		}
		$this->write('From: '.$this->from);
		$this->write('To: '.$to);
		$this->write('Date: '.date('r'));
		$this->write('Subject: '.$subject);
		foreach ($headers as $h)
		{
			$this->write($this->clean_str($h));
		}
		$this->write('');
		$this->write($body);
		$this->write('.');
		if ($this->read() != '250')
		if (substr($this->read(),0,2) != '354')
		{
			$this->reset();
			return false;
		}
		
		return true;
	}
	
	public function disconnect()
	{
		if (!$this->conn) return false;
		$this->write('QUIT');
		$this->read();
		$this->close();
	}
		
	
	private function close()
	{
		if (!$this->conn) return false;
		fclose($this->conn);
		$this->conn = false;
	}
	
	private function read()
	{
		if (!$this->conn) return false;
		do {
			$str = @fgets($this->conn);
			$this->log .= 'S '.$str;
		} while (substr($str,3,1) == '-');
		return substr($str,0,3);
	}
	
	private function write($str,$logstr=false)
	{
		if (!$this->conn) return false;
		$str.="\r\n";
		@fwrite($this->conn, $str);
		if ($logstr === false)
			$this->log .= 'C '.$str;
		else
			$this->log .= 'C '.$logstr."\r\n";
	}
	
	private function log($str, $setlasterror=false)
	{
		$this->log .= '* '.$str."\r\n";
		if ($setlasterror)
			$this->lasterror = $str;
	}
	
	private function clean_str($str,$body=false)
	{
		$out = '';
		$newline = false;
		$len = strlen($str);
		for ($i = 0; $i < $len; $i++)
		{
			if (ord($str[$i]) > ($body ? 0 : 31) and ord($str[$i]) < 127)
			{
				if ($str[$i]=='.' and $i>0 and $str[$i-1] == "\n")
					$out.='.';
				$out.=$str[$i];
				
			}
		}
		return $out;
	}
	
	private function reset()
	{
		$this->write('RSET');
		if ($this->read() != '250')
		{ 
			$this->log('RESET fehlgeschlagen, Verbindung wird neu aufgebaut.');
			$this->disconnect();
			return $this->connect($this->addr, $this->port, $this->enc, $this->auth, $this->cert, $this->user, $this->pass, $this->from);
		}
		return true;
	}
	
	public function getlog()
	{
		$temp = $this->log;
		$this->log = '';
		return $temp;
	}
	
	public function geterror()
	{
		return $this->lasterror;
	}
}

class mail_send_wrapper
{
	private $smtp_obj = FALSE;
	private $conn = FALSE;
	
	public function __destruct()
	{
		$this->disconnect();
	}
	
	public function getlog()
	{
		if ($this->smtp_obj)
			return $this->smtp_obj->getlog();
		else
			return false;
	}
	
	public function geterror()
	{
		if ($this->smtp_obj)
			return $this->smtp_obj->geterror();
		else
			return false;
	}

	public function connect($smtp=false)
	{
		if ($this->conn) return false;
		if (!$smtp)
		{
			$this->conn = true;
			$this->smtp_obj = false;
			
		}
		else
		{
			if (!$this->smtp_obj)
				$this->smtp_obj = new smtp_mail;
			$this->conn = $this->smtp_obj->connect(
				$smtp['addr'],
				$smtp['port'],
				$smtp['encryption'],
				$smtp['authentication'],
				$smtp['certcheck'],
				$smtp['username'],
				$smtp['password'],
				$smtp['sender']
				);
		}
		return $this->conn;
	}
	
	public function disconnect()
	{
		if (!$this->conn) return false;
		if ($this->smtp_obj)
		{
			$this->smtp_obj->disconnect();
		}
		$this->conn = false;
		return true;
	}
	
	public function send($to, $subject, $body, $quoted=false)
	{
		if (!$this->conn) return false;
		$headers = array('Content-Type: text/plain; charset=UTF-8');
		if ($quoted)
		{
			$body = quoted_printable_encode($body);
			$headers[] = 'Content-Transfer-Encoding: quoted-printable';
		}
		
		if (!$this->smtp_obj)
		{
			global $g_settings;
			$headers[] = 'From: '.$g_settings['phpmail_from'];
			$headers = implode(PHP_EOL, $headers);
			// zunächst alle Zeilenumbrüche in "\n" umwandeln,
			// dann "\n" in PHP_EOL umwandeln
			$body = str_replace(array("\r\n","\r","\n"),array("\n","\n",PHP_EOL),$body);
			$s = mail($to, $subject, $body, $headers);
		}
		else
		{
			$body = str_replace(array("\r\n","\r","\n"),array("\n","\n","\r\n"),$body);
			$s = $this->smtp_obj->send($to, $subject, $body, $headers);
		}
		return $s;
	}
}


