<?php
class url_check_curl_easy
{
	private $queue, $ch;
	
	function __construct()
	{
		$this->queue = array();
		$this->ch = curl_init();
		
		curl_setopt($this->ch, CURLOPT_CAINFO, CERTFILE);
		
		// HTTP 4xx und 5xx nicht als Fehler werten
		curl_setopt($this->ch, CURLOPT_FAILONERROR, false);
		
		// Header zurückgeben
		curl_setopt($this->ch, CURLOPT_HEADER, true);
		
		// User Agent setzen
		curl_setopt($this->ch, CURLOPT_USERAGENT, 'Rosmarin Servertest '.VERSION);
		
		// keine direkte Ausgabe, Ergebnis als String zurückgeben
		curl_setopt($this->ch, CURLOPT_RETURNTRANSFER, true);
		
		// Location-Headern nicht folgen
		curl_setopt($this->ch, CURLOPT_FOLLOWLOCATION, false);
		// Die Curl-interne Verarbeitung von Redirects hat einige Probleme:
		// * Wenn die maximale Anzahl an Weiterleitungen erreicht ist, liefert
		//   Curl einen Fehler statt des Seiteninhalts
		// * Weiterleitungen lassen sich erst ab PHP 5.2.10 und Curl 7.19.4
		//   auf http(s) beschränken 
		// * mit safe_mode oder open_basedir sind Weiterleitungen erst ab
		//   PHP 5.6 und Curl 7.19.4 möglich
	}
	
	function __destruct()
	{
		curl_close($this->ch);
	}

	function add($id, $url, $timeout, $maxredirects, $maxretries, $expected, $check_statuscode, $check_content, $certcheck)
	{
		$this->queue[] = array(
			'url' => $url,
			'id' => $id,
			'timeout' => $timeout,
			'maxredirects' => $maxredirects,
			'redirects' => 0,
			'maxretries' => $maxretries,
			'retries' => 0,
			'expected' => $expected,
			'check_statuscode' => $check_statuscode,
			'check_content' => $check_content,
			'cert' => $certcheck,
			'result' => array(),
			);
	}
	
	function exec()
	{
		$matches = array();
		foreach ($this->queue as &$task)
		{
			$opt = array(
				CURLOPT_URL => $task['url'],
				CURLOPT_TIMEOUT => $task['timeout'],
				CURLOPT_SSL_VERIFYHOST => $task['cert'] ? 2 : 0,
				CURLOPT_SSL_VERIFYPEER => $task['cert'],
				);
			curl_setopt_array($this->ch, $opt);
			$log = '';
			
			do
			{
				$again = false;
				$result = false;
				$data = curl_exec($this->ch);
				if ($data === false)
				{
					$result = curl_error($this->ch);
				}
				else
				{
					// Header vom Body trennen
					$hend = strpos($data,"\r\n\r\n")+4;
					if ($hend !== false)
					{
						$header = substr($data,0,$hend);
						$body = substr($data,$hend);
						$log.=$header;
					}
					else
					{
						$result = 'malformed http';
						$log.= $data;
					}
					
					
					// Weiterleitung (Location-Header)
					if ( !$result and in_array(curl_getinfo($this->ch,CURLINFO_HTTP_CODE), array(301,302,303,307,308))
						and preg_match('/\r\nLocation:(.*)\r\n/i', $header, $matches) )
					{
						if ($task['maxredirects'] > $task['redirects'])
						{
							$redir_url = trim($matches[1]);
							
							// Aus Sicherheitsgründen nur zu http(s) weiterleiten
							if (strtolower(substr($redir_url,0,4)) == 'http')
							{
								$again = true;
								$task['redirects']++;
								curl_setopt($this->ch, CURLOPT_URL, $redir_url);
								continue;
							}
							else
							{
								$result = 'Weiterleitung nur zu http(s) erlaubt';
							}
						}
						else
						{
							$log.= '*** Maximum von '.$task['maxredirects'].' Weiterleitungen erreicht. ***'."\r\n\r\n";
						}
					}
					
					
					// HTTP-Anfrage erfolgreich
					if (!$result)
					{
						$log.=$body;
						if( $task['check_content'])
						{
							if ($task['check_statuscode'])
								$result= curl_getinfo($this->ch,CURLINFO_HTTP_CODE).' '.curl_getinfo($this->ch,CURLINFO_CONTENT_TYPE).' '.md5($body);
							else
								$result= curl_getinfo($this->ch,CURLINFO_CONTENT_TYPE).' '.md5($body);
						}
						else
						{
							if ($task['check_statuscode'])
								$result= curl_getinfo($this->ch,CURLINFO_HTTP_CODE);
							else
								$result= 'OK';
						}
					}
				}
				
				if ($task['retries'] > 0)
				{
					$result.= sprintf(' (%d. Versuch)', $task['retries']+1);
				}
				$task['result'][] = $result;
				
				// Wiederholung, falls nicht das gewünschte Ergebnis erzielt wurde
				if ($task['expected'] != $result)
				{
					if ($task['maxretries'] > $task['retries'])
					{
						$task['retries']++;
						$log .= "\r\n\r\n*** Server lieferte nicht die erwartete Antwort. Starte Versuch ".($task['retries']+1).". ***\r\n\r\n";
						$task['redirects'] = 0;
						curl_setopt($this->ch, CURLOPT_URL, $task['url']);
						$again = true;
					}
					else
					{
						$log .= "\r\n\r\n*** Server lieferte nicht die erwartete Antwort. Maximum von ".$task['maxretries']." Wiederholungen erreicht. ***\r\n\r\n";
					}
				}
			} while ($again);
			
			$task = array(
				'id' => $task['id'],
				'result' => $task['result'],
				'log' => $log,
				);
		}
		unset($task);
		$temp_queue = $this->queue;
		$this->queue = array();
		return $temp_queue;
	}
}

if (!defined('WEBLIB')) define('WEBLIB','url_check_curl_easy');
return 'url_check_curl_easy';
