<?php

// Benötigt curl_multi_info_read()
// Funktioniert erst ab 5.2
// (PHP Bug 36537)

class url_check_curl_multi
{
	private $queue, $mh;
		
	function __construct()
	{
		$this->queue = array();
		$this->mh = curl_multi_init();
	}
	
	function __destruct()
	{
		curl_multi_close($this->mh);
	}
	
	function add($id, $url, $timeout, $maxredirects, $maxretries, $expected, $check_statuscode, $check_content, $certcheck)
	{
		$ch = curl_init($url);
		$opt = array(
			CURLOPT_CAINFO => CERTFILE,
			CURLOPT_FAILONERROR => false,
			CURLOPT_HEADER => true,
			CURLOPT_USERAGENT => 'Rosmarin Servertest '.VERSION,
			CURLOPT_RETURNTRANSFER => true,
			CURLOPT_TIMEOUT => $timeout,
			CURLOPT_SSL_VERIFYHOST => $certcheck ? 2 : 0,
			CURLOPT_SSL_VERIFYPEER => $certcheck,
			CURLOPT_FOLLOWLOCATION => false,
			);
		curl_setopt_array($ch, $opt);
		curl_multi_add_handle($this->mh,$ch);
		
		$this->queue[(int)$ch] = array(
			'id' => $id,
			'url' => $url,
			'maxredirects' => $maxredirects,
			'redirects' => 0,
			'maxretries' => $maxretries,
			'retries' => 0,
			'expected' => $expected,
			'check_statuscode' => $check_statuscode,
			'check_content' => $check_content,
			'log' => '',
			'result' => array(),
			);
	}
	
	function exec()
	{
		$running = 0;
		$matches = array();
		while (true)
		{
			while (curl_multi_exec($this->mh, $running) == CURLM_CALL_MULTI_PERFORM);
			while ($done = curl_multi_info_read($this->mh))
			{
				$ch = $done['handle'];
				$task = &$this->queue[(int)$ch];
				$redir_url = false;
				$result = false;
				if ($done['result'] != CURLE_OK)
				{
					$result = curl_error($ch);
					if (empty($result))
					{
						// http://bugs.php.net/bug.php?id=65684
						$result = 'Error '.$done['result'].'. See http://curl.haxx.se/libcurl/c/libcurl-errors.html';
					}
				}
				else
				{
					$data = curl_multi_getcontent($ch);
					
					
					// Header vom Body trennen
					$hend = strpos($data,"\r\n\r\n")+4;
					if ($hend !== false)
					{
						$header = substr($data,0,$hend);
						$body = substr($data,$hend);
						$task['log'].=$header;
					}
					else
					{
						$result = 'malformed http';
						$task['log'] = $data;
					}
					
					
					// Weiterleitung (Location-Header)
					if ( !$result and in_array(curl_getinfo($ch,CURLINFO_HTTP_CODE), array(301,302,303,307,308))
						and preg_match('/\r\n(Location:)(.*?)\r\n/i', $header, $matches) )
					{
						if ($task['maxredirects'] > $task['redirects'])
						{
							$redir_url = trim($matches[2]);
							if (strtolower(substr($redir_url,0,4)) != 'http')
							{
								$redir_url = false;
								$result = 'Weiterleitung nur zu http(s) erlaubt';
							}
						}
						else
						{
							$task['log'].= '*** Maximum von '.$task['maxredirects'].' Weiterleitungen erreicht. ***'."\r\n\r\n";
						}
					}
					
					
					// HTTP-Anfrage erfolgreich
					if (!$result and !$redir_url)
					{
						$task['log'].=$body;
						if( $task['check_content'])
						{
							if ($task['check_statuscode'])
								$result= curl_getinfo($ch,CURLINFO_HTTP_CODE).' '.curl_getinfo($ch,CURLINFO_CONTENT_TYPE).' '.md5($body);
							else
								$result= curl_getinfo($ch,CURLINFO_CONTENT_TYPE).' '.md5($body);
						}
						else
						{
							if ($task['check_statuscode'])
								$result= curl_getinfo($ch,CURLINFO_HTTP_CODE);
							else
								$result= 'OK';
						}
					}
				}
				curl_multi_remove_handle($this->mh,$ch);
				
				// Weiterleitung (Location-Header)
				if ($redir_url)
				{
					$task['redirects']++;
					curl_setopt($ch, CURLOPT_URL, $redir_url);
					curl_multi_add_handle($this->mh,$ch);
					continue 2;
				}
				
				if ($task['retries'] > 0)
				{
					$result.= sprintf(' (%d. Versuch)', $task['retries']+1);
				}
				$task['result'][] = $result;
				
				// Wiederholung, falls nicht das gewünschte Ergebnis erzielt wurde
				if ($task['expected'] != $result)
				{
					if ($task['maxretries'] > $task['retries'])
					{
						$task['retries']++;
						$task['log'] .= "\r\n\r\n*** Server lieferte nicht die erwartete Antwort. Starte Versuch ".($task['retries']+1).". ***\r\n\r\n";
						$task['redirects'] = 0;
						curl_setopt($ch, CURLOPT_URL, $task['url']);
						curl_multi_add_handle($this->mh,$ch);
						continue 2;
					}
					else
					{
						$task['log'] .= "\r\n\r\n*** Server lieferte nicht die erwartete Antwort. Maximum von ".$task['maxretries']." Wiederholungen erreicht. ***\r\n\r\n";
					}
				}
				
				// Task abgearbeitet
				else
				{
					curl_close($ch);
					$task = array(
						'id' => $task['id'],
						'result' => $task['result'],
						'log' => $task['log'],
						);
				}
			}
			if ($running == 0) break;
			curl_multi_select($this->mh);
		}
		$temp_queue = $this->queue;
		$this->queue = array();
		return $temp_queue;
	}
}
if (!defined('WEBLIB')) define('WEBLIB','url_check_curl_multi');
return 'url_check_curl_multi';
