<?php

require_once('error_buffer.php');

class url_check_simple
{
	private $queue;
	
	function __construct()
	{
		$this->queue = array();
	}
	
	function add($id, $url, $timeout, $maxredirects, $maxretries, $expected, $check_statuscode, $check_content, $certcheck)
	{
		$this->queue[] = array('url' => $url,
			'id' => $id,
			'timeout' => $timeout,
			'maxredirects' => $maxredirects,
			'redirects' => 0,
			'maxretries' => $maxretries,
			'retries' => 0,
			'expected' => $expected,
			'check_statuscode' => $check_statuscode,
			'check_content' => $check_content,
			'cert' => $certcheck,
			'result' => array(),
			);
	}
	
	function exec()
	{
		$matches = array();
		foreach ($this->queue as &$task)
		{
			$opt = array(
				'http' => array(
					'max_redirects' => 0,
					// Die interne Verarbeitung von Redirects hat einige Probleme:
					// * CN_match kann nicht an die neue URL angepasst werden
					
					'ignore_errors' => true,
					'timeout' => $task['timeout'],
					'user_agent' => 'Rosmarin Servertest '.VERSION,
					),
				'ssl' => array(
					'verify_peer' => (bool)$task['cert'],
					'cafile' => CERTFILE,
					),
				);
			
			$log = '';
			do
			{
				$again = false;
				$opt['ssl']['CN_match'] = substr($task['url'],strpos($task['url'],'://')+3);
				$context = stream_context_create($opt);
				set_error_handler('error_buffer',E_WARNING);
				$data = @file_get_contents($task['url'],false,$context);
				restore_error_handler();
				if ($data === false)
				{
					$result = error_buffer();
					$result = trim_errormsg($result[0]);
				}
				else
				{
					$task['contenttype'] = 'unknown';
					$task['statuscode'] = '000';
					$task['redirect_location'] = '';
					
					foreach ($http_response_header as $header)
					{
						if (strtolower(substr($header, 0, 13)) == 'content-type:')
						{
							$task['contenttype'] = trim(substr($header, 13));
						}
						if (substr($header, 0, 5) == 'HTTP/')
						{
							$task['statuscode'] = substr($header, 9,3);
						}
						if (strtolower(substr($header, 0, 9)) == 'location:')
						{
							$task['redirect_location'] = trim(substr($header, 9));
						}
						$log.=$header."\r\n";
					}
					$log .= "\r\n";
					
					// Weiterleitung (Location-Header)
					if ( in_array($task['statuscode'], array(301,302,303,307,308)) and $task['redirect_location'] )
					{
						if ($task['maxredirects'] > $task['redirects'])
						{
							$redir_url = trim($task['redirect_location']);
							
							// Aus Sicherheitsgründen nur zu http(s) weiterleiten
							if (strtolower(substr($redir_url,0,4)) == 'http')
							{
								$again = true;
								$task['redirects']++;
								$task['url'] = $redir_url;
								continue;
							}
							else
							{
								$result = 'Weiterleitung nur zu http(s) erlaubt';
							}
						}
						else
						{
							$log.= '*** Maximum von '.$task['maxredirects'].' Weiterleitungen erreicht. ***'."\r\n\r\n";
						}
					}
					
					$log .= $data;
					if ($task['check_content'])
					{
						if ($task['check_statuscode'])
							$result= $task['statuscode'].' '.$task['contenttype'].' '.md5($data);
						else
							$result= $task['contenttype'].' '.md5($data);
					}
					else
					{
						if ($task['check_statuscode'])
							$result= $task['statuscode'];
						else
							$result= 'OK';
					}
				}
				
				if ($task['retries'] > 0)
				{
					$result.= sprintf(' (%d. Versuch)', $task['retries']+1);
				}
				$task['result'][] = $result;
				
				// Wiederholung, falls nicht das gewünschte Ergebnis erzielt wurde
				if ($task['expected'] != $result)
				{
					if ($task['maxretries'] > $task['retries'] and $task['expected'] != $result)
					{
						$task['retries']++;
						$log .= "\r\n\r\n*** Server lieferte nicht die erwartete Antwort. Starte Versuch ".($task['retries']+1).". ***\r\n\r\n";
						$task['redirects'] = 0;
						$again = true;
					}
					else
					{
						$log .= "\r\n\r\n*** Server lieferte nicht die erwartete Antwort. Maximum von ".$task['maxretries']." Wiederholungen erreicht. ***\r\n\r\n";
					}
				}
			} while ($again);
			
			$task = array(
				'id' => $task['id'],
				'result' => $task['result'],
				'log' => $log,
				);
		}
		unset($task);
		$temp_queue = $this->queue;
		$this->queue = array();
		return $temp_queue;
	}
}

if (!defined('WEBLIB')) define('WEBLIB','url_check_simple');
return 'url_check_simple';
